#!/usr/bin/env python3

"""
Skeleton for homework 6 part 3
"""

import sys

from pathlib import Path

import requests


SCIPER_FILE = ".sciper"
SERVER_BASE_URL = "https://com301-cert.epfl.ch"


def homework6_part3(server_list):
    """Homework 6 part 3
    To complete!
    """

    cert_validities = list()

    # TODO: Probably you want to look at each server.
    server = server_list[0]
    host = server["host"]
    port = server["port"]

    # TODO: A quite stupid validation. You probably want to do something
    # a little bit smarter than this.
    import random
    cert_is_valid = random.choice((True, False))

    server_validity = {
        "host": host,
        "port": port,
        "valid": cert_is_valid
    }

    # TODO: Make sure cert_validities contains all servers
    cert_validities.append(server_validity)

    return cert_validities


def load_sciper():
    """Load the SCIPER number from a dot file in the home directory of the user."""

    path = Path.home() / SCIPER_FILE

    if path.exists():
        if path.is_file():
            with path.open("r") as sciper_file:
                sciper = int(sciper_file.read().strip())
            return sciper
        else:
            raise Exception("Can not retrieve your sciper from \"{}\"!".format(path))
    else:
        err_str = "Your SCIPER is not configured!\n" \
            "Please set your SCIPER with:\n" \
            "helper config --sciper <your sciper>"
        raise Exception(err_str.format(sys.argv[0]))


def retrieve_server_list(sciper):
    """Retrieve the server list for the user to validate."""

    endpoint = "/server-list"

    content = {
        "sciper": sciper
    }

    url = "{}{}".format(SERVER_BASE_URL, endpoint)

    response = requests.get(url, json=content)

    if response.status_code != 200:
        raise Exception("An error occurred while trying to fetch some data from the server.")

    return response.json()


class ValidationResponse():
    """Response for validation."""

    def __init__(self, success, msg="", token=""):
        self.success = success
        self.msg = msg
        self.token = token

    def __str__(self):
        if self.success:
            return "Congratulations! :-D\nYour token is: {}".format(self.token)

        return "Unfortately, your response is wrong. :-(\n{}".format(self.msg)


def validate_part3(sciper, validity_list):
    """Validate the list of servers."""

    endpoint = "/validate-part3"

    content = {
        "sciper": sciper,
        "validity_list": validity_list
    }

    url = "{}{}".format(SERVER_BASE_URL, endpoint)

    response = requests.post(url, json=content)

    if response.status_code != 200:
        raise Exception("An error occurred on the server while validating your answer.")

    response_dict = response.json()

    return ValidationResponse(**response_dict)


def main():
    """Entrypoint of the program."""

    # pylint: disable=broad-except

    sciper = load_sciper()
    print("SCIPER: {}".format(sciper))

    try:
        server_list = retrieve_server_list(sciper)
    except Exception as err:
        sys.stderr.write("{}\n".format(err))
        sys.stderr.flush()
        sys.exit(1)

    validity_list = homework6_part3(server_list)

    response = validate_part3(sciper, validity_list)
    print(response)


if __name__ == "__main__":
    main()
